@@ -30,7 +30,7 @@
 #define PYTHON_CONVERT_H
 
 #include <Python.h>
-#include "types.h"
+#include <igraph/types.h>
 #include "graphobject.h"
 
 typedef enum { IGRAPHMODULE_TYPE_INT=0, IGRAPHMODULE_TYPE_FLOAT }
@@ -47,6 +47,7 @@ int igraphmodule_PyObject_to_bliss_sh_t(PyObject *o, igraph_bliss_sh_t *result);
 int igraphmodule_PyObject_to_vconn_nei_t(PyObject *o, igraph_vconn_nei_t *result);
 int igraphmodule_PyObject_to_neimode_t(PyObject *o, igraph_neimode_t *result);
 int igraphmodule_PyObject_to_adjacency_t(PyObject *o, igraph_adjacency_t *result);
+int igraphmodule_PyObject_to_connectedness_t(PyObject *o, igraph_connectedness_t *result);
 int igraphmodule_PyObject_to_degseq_t(PyObject *o, igraph_degseq_t *result);
 
 int igraphmodule_PyObject_to_integer_t(PyObject *object, igraph_real_t *v);
@@ -60,6 +61,8 @@ PyObject* igraphmodule_vector_t_to_PyList(igraph_vector_t *v, igraphmodule_conv_
 PyObject* igraphmodule_vector_t_to_PyTuple(igraph_vector_t *v);
 int igraphmodule_attrib_to_vector_t(PyObject *o, igraphmodule_GraphObject *self,
   igraph_vector_t **vptr, int attr_type);
+int igraphmodule_attrib_to_vector_bool_t(PyObject *o, igraphmodule_GraphObject *self,
+  igraph_vector_bool_t **vptr, int attr_type);
 PyObject* igraphmodule_vector_t_pair_to_PyList(igraph_vector_t *v1,
 					       igraph_vector_t *v2);
 PyObject* igraphmodule_vector_t_to_PyList_pairs(igraph_vector_t *v);
